LoadPackage("atlasrep");

### This function returns the set of conjugacy classes containing the products
### rs, where r is in the conjugacy class <a> and s is in <b>
squares := function(ct, a, b)
  local rsq, j;
  rsq := [];
  for j in [1..NrConjugacyClasses(ct)] do
    if ClassMultiplicationCoefficient(ct, a, b, j) > 0 then
      Add(rsq, PowerMap(ct, 2)[j]);
    fi;
  od;
  return Set(rsq);
end;

### This function prints the possible classes of the character table <ct2>
### of the subgroup contained in the class <c> of the 
### group with character table <ct1>
look_for_fusions := function(ct2, ct1, c)
  local i, r, fus;
  r := [];
  fus := FusionConjugacyClasses(ct2, ct1);
  for i in [1..Size(fus)] do
    if fus[i] = c then
      Add(r, i);
    fi;
  od;
  return r;
end;

groupname := "B";
subgroup := "2.2E6(2).2";

LogTo("step5.log");

Print("Fusion of conjugacy classes from ", subgroup, " into ", groupname, "\n");

ct1 := CharacterTable(groupname);
ct2 := CharacterTable(subgroup);

fus := FusionConjugacyClasses(ct2, ct1);
if not fus = fail then
  for t in [1..Size(fus)] do
    Print(ClassNames(ct2)[t], " --> ", ClassNames(ct1)[fus[t]], "\n");
  od;
else
  for fus in PossibleClassFusions(ct2, ct1) do
    for t in [1..Size(fus)] do
      Print(ClassNames(ct2)[t], " --> ", ClassNames(ct1)[fus[t]], ";\n");
    od;
    Print("--\n");
  od;
fi;

Print("--\n");
for c in [ct1.16c, ct1.16d] do
  Print("The conjugacy class ", ClassNames(ct1)[c], "\n");
  Print("Classes of ", Identifier(ct2), " contained in ", ClassNames(ct1)[c], ":\n");
  
  f := look_for_fusions(ct2, ct1, c);
  Print(List(f, x->ClassNames(ct2)[x]), "\n");
  a := f[1];
  b := f[2];
  Print("Let r in ", ClassNames(ct2)[a], ", and s in ", ClassNames(ct2)[b], ". ");
  Print("The orders of (rs)^2 are:\n");
  Display(squares(ct2, a, b));
  Print("==\n");
od;

