LoadPackage("atlasrep");
LoadPackage("io");

checkD_random := function(gr, x, n)
  local sg, i, t, y;
  Print("Trying with n=", n, "\n");
  for i in [1..n] do
  	t := Random(gr);
    y := t*x*Inverse(t);
    if not (x*y)^2 = (y*x)^2 then 
      sg := Subgroup(gr, [x,y]);
      if IsConjugate(sg, x, y) = false then
        return [x,y];
      fi;
    fi;
  od;
  return fail;
end;

checkD := function(gr, c)
  local sg, x, y;
  x := Representative(c);
  for y in c do
    if not (x*y)^2 = (y*x)^2 then 
      sg := Subgroup(gr, [x,y]);
      if IsConjugate(sg, x, y) = false then
        return [x,y];
      fi;
    fi;
  od;
  return fail;
end;

x := [];
g := [];

M := 18;
file := Concatenation("max", String(M));

LogTo("M18.log");

Print("Group Fi24'.2, maximal: ", file, "\n\n");

_gr := DirectProduct(SymmetricGroup(6), AtlasGroup("L2(8).3"));#AtlasGroup("O10-(2).2"); 
_cc := ConjugacyClasses(_gr);

bg := BaseOfGroup(_gr);
Print("Base of Group: ", bg, "\n--\n");

f := IO_File("M18.rec", "w");
IO_WriteLine(f,"### The format for classinfo is [ r, s, order, comment ]");
IO_WriteLine(f,"data := rec(");
IO_WriteLine(f,"  group := \"", file, "\",");
IO_WriteLine(f,"  base := ", bg, ",");
IO_WriteLine(f,"  generators := \"", GeneratorsOfGroup(_gr), "\",");
IO_WriteLine(f,"  comments := \"Maximal subgroups\",");
IO_WriteLine(f,"  classinfo := [");

for c in _cc do
  g := Representative(c);

  if not Order(g) in [18] then
    continue;
  fi;

  Print("Representative of order ", Order(g), "\n");
  rs := checkD_random(_gr, g, 50000); 
  if not rs = fail then
    Print(" Conjugacy class of size: ", Size(c), "\n");
    Print(" Centralizer of order: ", Size(_gr)/Size(c), "\n");
    Print(" r:=", OnTuples(bg, rs[1]), ";\n");
    Print(" s:=", OnTuples(bg, rs[2]), ";\n");
    IO_WriteLine(f, "    [ ", OnTuples(bg, rs[1]), ", ", OnTuples(bg, rs[2]), ", ", Order(g), ", \"D\" ],");
    Print("Removed!\n");
    Print("--\n");
  fi;
od;

IO_WriteLine(f, "  ]");
IO_WriteLine(f, ");");
IO_Flush(f);
IO_Close(f);


