LoadPackage("atlasrep");
LoadPackage("io");

checkD_random := function(gr, x, n)
  local sg, i, t, y;
  Print("Trying with n=", n, "\n");
  for i in [1..n] do
  	t := Random(gr);
    y := t*x*Inverse(t);
    if not (x*y)^2 = (y*x)^2 then 
      sg := Subgroup(gr, [x,y]);
      if IsConjugate(sg, x, y) = false then
        return [x,y];
      fi;
    fi;
  od;
  return fail;
end;

checkD := function(gr, c)
  local sg, x, y;
  x := Representative(c);
  for y in c do
    if not (x*y)^2 = (y*x)^2 then 
      sg := Subgroup(gr, [x,y]);
      if IsConjugate(sg, x, y) = false then
        return [x,y];
      fi;
    fi;
  od;
  return fail;
end;

x := [];
g := [];
n := 0;

name := "M22.2";

LogTo(Concatenation(name, ".log"));

M := Size(Maxes(CharacterTable(name)));

gr := AtlasGroup(name);
cc := ConjugacyClasses(gr);
ct := CharacterTable(name);

Print("Group: ", name, "\n");
Print("Number of maximal subgroups: ", M, "\n");

ns := ClassPositionsOfNormalSubgroups(ct);
cl := Filtered([1..NrConjugacyClasses(ct)], x->not x in ns[2]);
Print("Conjugacy classes to study:\n");
for x in cl do
  Print(ClassNames(ct, "atlas")[x], ", ");
od;
Print("\n");
Display(List(cl, x->[ClassNames(ct, "atlas")[x], SizesConjugacyClasses(ct)[x], SizesCentralizers(ct)[x]]));
Print("==\n");

Print("Connecting group and character table: ", ConnectGroupAndCharacterTable(gr, ct), "\n");

bg := BaseOfGroup(gr);
Print("Base of Group: ", bg, "\n--\n");

f := IO_File(Concatenation(name, ".rec"), "w");
IO_WriteLine(f,"### The format for classinfo is [ r, s, order, comment ]");
IO_WriteLine(f,"data := rec(");
IO_WriteLine(f,"  base := ", bg, ",");
IO_WriteLine(f,"  group := \"", name, "\",");
IO_WriteLine(f,"  comments := \"Maximal subgroups\",");
IO_WriteLine(f,"  classinfo := [");

rem := [];

M := Size(Maxes(CharacterTable(name)));

for i in [1..M] do

  _gr := Group(AtlasGenerators(name, 1, M+1-i).generators);
  _cc := ConjugacyClasses(_gr);

  # For  an  ordinary character table <tbl> with known underlying group $G$,
  # IdentificationOfConjugacyClasses returns a list of positive integers that
  # contains at position i the position of the i-th conjugacy class of <tbl>
  # in the list ConjugacyClasses(G).
  for c in _cc do
    if Size(cl) > Size(rem) then
      g := Representative(c);
      n := Position(IdentificationOfConjugacyClasses(ct), Position(List(cc, x->g in x), true));
      if n in cl and not n in rem then
        Print("Checking: ", ClassNames(ct, "atlas")[n], " (maximal subgroup ", M+1-i, ")\n");
        rs := checkD(_gr, c); 
        if not rs = fail then
        	Print("r:=", OnTuples(bg, rs[1]), ";\n");
        	Print("s:=", OnTuples(bg, rs[2]), ";\n");
          IO_WriteLine(f, "    [ ", OnTuples(bg, rs[1]), ", ", OnTuples(bg, rs[2]), ", ", Order(g), ", \"D\" ],");
          Print("Removed: ", ClassNames(ct, "atlas")[n], "\n");
          Add(rem, n);
          Print("--\n");
        fi;
      fi;
    fi;
  od;
od;

a := List(cl, x->ClassNames(ct,"atlas")[x]);
b := List(rem, x->ClassNames(ct,"atlas")[x]);

diff := Difference(a, b);

Print("Summary:\n");
Print("To study: ", a, "\n");
Print("Removed: ", b, "\n");
Print("Not of type D: ", diff, "\n");
Print("==\n");

for x in List(diff, x->Position(ClassNames(ct, "atlas"), x)) do
  Print("Checking: ", ClassNames(ct, "atlas")[x], " (in ", name, ")\n");
  c := cc[IdentificationOfConjugacyClasses(ct)[x]]; 
  rs := checkD(gr, c);
  if not rs = fail then
    Print("r:=", OnTuples(bg, rs[1]), ";\n");
    Print("s:=", OnTuples(bg, rs[2]), ";\n");
    IO_WriteLine(f, "    [ ", OnTuples(bg, rs[1]), ", ", OnTuples(bg, rs[2]), ", ", Order(g), ", \"D\" ],");
    Print("Removed: ", ClassNames(ct, "atlas")[n], "\n");
    Add(rem, n);
    Print("--\n");
  else
    Print("Fail!\n");
    IO_WriteLine(f, "    [ ", OnTuples(bg, Representative(c)), ", ", ", ", Order(g), ", \"fail\" ],");
  fi;
od;
 
IO_WriteLine(f, "  ]");
IO_WriteLine(f, ");");
IO_Flush(f);
IO_Close(f);


